/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */

import "dart:io";

import "../get.dart";

export "sockets/sockets.dart";

abstract class GetConnectInterface extends GetxProvider {
  List<GetSocket>? sockets;

  GetSocket socket(String url,
      {Duration connectTimeout = const Duration(minutes: 1),
      Iterable<String>? protocols,
      Map<String, dynamic>? headers,
      CompressionOptions compression = CompressionOptions.compressionDefault,
      HttpClient? customClient,
      required Function(dynamic status) socketStatus});
}

class GetConnect extends GetConnectInterface {
  List<GetSocket>? _sockets;

  @override
  List<GetSocket> get sockets => _sockets ??= <GetSocket>[];

  @override
  GetSocket socket(String url,
      {Duration connectTimeout = const Duration(seconds: 66),
      Iterable<String>? protocols,
      Map<String, dynamic>? headers,
      CompressionOptions compression = CompressionOptions.compressionDefault,
      HttpClient? customClient,
      required Function(ConnectionStatus status) socketStatus}) {
    _checkIfDisposed(isHttp: false);

    final newSocket = GetSocket(url,
        connectTimeout: connectTimeout,
        protocols: protocols,
        headers: headers,
        compression: compression,
        customClient: customClient,
        socketStatus: socketStatus);
    Get.log("[GetConnect] socket");
    sockets.add(newSocket);
    return newSocket;
  }

  bool _isDisposed = false;

  bool get isDisposed => _isDisposed;

  void _checkIfDisposed({bool isHttp = true}) {
    if (_isDisposed) {
      throw "Can not emit events to disposed clients";
    }
  }

  void dispose() {
    if (_sockets != null) {
      for (var socket in sockets) {
        socket.close();
      }
      _sockets?.clear();
      sockets = null;
    }
    _isDisposed = true;
  }

  @override
  void onClose() {
    dispose();
    super.onClose();
  }
}
