/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */

import "dart:async";
import "dart:io";

import "package:get/get.dart";
import "package:get/get_connect/socket_channel/io.dart";
import "package:get/get_connect/sockets/src/socket_notifier.dart";

@Deprecated("当前方法基于web_socket_channel实现")
class NewBaseWebSocket {
  String url;
  IOWebSocketChannel? socket;
  Iterable<String>? protocols;
  Map<String, dynamic>? headers;
  CompressionOptions compression = CompressionOptions.compressionDefault;
  HttpClient? customClient;
  SocketNotifier? socketNotifier = SocketNotifier();
  bool isDisposed = false;
  Duration connectTimeout = const Duration(seconds: 90);
  bool allowSelfSigned;
  ConnectionStatus connectionStatus = ConnectionStatus.none;
  Function(ConnectionStatus status) socketStatus;

  NewBaseWebSocket(this.url,
      {this.connectTimeout = const Duration(minutes: 1),
      this.allowSelfSigned = false,
      this.protocols,
      this.headers,
      this.compression = CompressionOptions.compressionDefault,
      this.customClient,
      required this.socketStatus});

  /// 连接
  Future connect() async {
    Get.log("[BaseWebSocket] connect = 0");
    if (connectionStatus != ConnectionStatus.received &&
        connectionStatus != ConnectionStatus.connecting &&
        connectionStatus != ConnectionStatus.connected &&
        connectionStatus != ConnectionStatus.sended &&
        connectionStatus != ConnectionStatus.closing) {
      if (isDisposed) {
        socketNotifier = SocketNotifier();
      }
      Get.log("[BaseWebSocket] connect");
      try {
        _setStatus(ConnectionStatus.connecting);
        socket = IOWebSocketChannel.connect(Uri.parse(url),
            headers: headers,
            protocols: protocols,
            connectTimeout: connectTimeout);
        await socket?.ready;
        _setStatus(ConnectionStatus.connected);
        Get.log("[BaseWebSocket] connected");
        socket?.stream.listen((data) {
          Get.log("[WS-Core] 收到消息 👉 $data");
          socketNotifier!.notifyData(data);
        }, onError: (err) {
          Get.log("[WS-Core] 监听异常 👉 $err");
          socketNotifier!.notifyError(Close(err.toString(), 1005));
        }, onDone: () {
          Get.log("[WS-Core] 关闭 websocket");
          _setStatus(ConnectionStatus.closed);
          socketNotifier?.notifyClose(
              Close(socket?.closeReason ?? "CLOSE", socket?.closeCode ?? 205));
        }, cancelOnError: true);
      } on SocketException catch (e) {
        Get.log("[BaseWebSocket] connected error SocketException");
        _setStatus(ConnectionStatus.closed);
        socketNotifier?.notifyError(
            Close(e.osError?.message ?? "ERR", e.osError?.errorCode ?? 500));
        return;
      }
    }
    return;
  }

  /// 发送消息
  void send(dynamic data) async {
    Get.log("[BaseWebSocket] send");
    if (connectionStatus == ConnectionStatus.closed) {
      _setStatus(ConnectionStatus.sending);
      await connect();
    }

    if (socket != null) {
      Get.log("[BaseWebSocket] RealSend");
      _setStatus(ConnectionStatus.sended);
      socket!.sink.add(data);
    }
  }

  /// 关闭连接
  void close([int? status, String? reason]) {
    Get.log("[BaseWebSocket] close");
    _setStatus(ConnectionStatus.closing);
    socket?.sink.close(status, reason);
    _setStatus(ConnectionStatus.closed);
  }

  void on(String event, MessageSocket message) {
    socketNotifier!.addEvents(event, message);
  }

  void onClose(CloseSocket fn) {
    socketNotifier!.addCloses(fn);
  }

  void onError(CloseSocket fn) {
    socketNotifier!.addErrors(fn);
  }

  void onMessage(MessageSocket fn) {
    socketNotifier!.addMessages((msg) {
      _setStatus(ConnectionStatus.received);
      fn(msg);
    });
  }

  void _setStatus(ConnectionStatus status) {
    connectionStatus = status;
    socketStatus(status);
  }
}
