/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */

import "dart:async";
import "dart:convert";
import "dart:io";
import "dart:math";

import "package:get/get_connect/sockets/src/sockets_enum.dart";

import "../../../get_core/get_core.dart";
import "socket_notifier.dart";

@Deprecated("当前方法基于Flutter默认实现")
class BaseWebSocket {
  String url;
  WebSocket? socket;
  Iterable<String>? protocols;
  Map<String, dynamic>? headers;
  CompressionOptions compression = CompressionOptions.compressionDefault;
  HttpClient? customClient;
  SocketNotifier? socketNotifier = SocketNotifier();
  bool isDisposed = false;
  bool allowSelfSigned;
  ConnectionStatus connectionStatus = ConnectionStatus.none;
  Function(ConnectionStatus status) socketStatus;

  BaseWebSocket(this.url,
      {this.allowSelfSigned = false,
      this.protocols,
      this.headers,
      this.compression = CompressionOptions.compressionDefault,
      this.customClient,
      required this.socketStatus});

  Future connect() async {
    Get.log("[BaseWebSocket] connect = 0");
    if (connectionStatus != ConnectionStatus.received &&
        connectionStatus != ConnectionStatus.connecting &&
        connectionStatus != ConnectionStatus.connected &&
        connectionStatus != ConnectionStatus.sended &&
        connectionStatus != ConnectionStatus.closing) {
      if (isDisposed) {
        socketNotifier = SocketNotifier();
      }
      try {
        Get.log("[BaseWebSocket] connect");
        _setStatus(ConnectionStatus.connecting);
        socket = allowSelfSigned
            ? await _connectForSelfSignedCert(url)
            : await WebSocket.connect(url,
                protocols: protocols,
                headers: headers,
                compression: compression,
                customClient: customClient);

        //socket!.pingInterval = ping;
        _setStatus(ConnectionStatus.connected);
        Get.log("[BaseWebSocket] connected");
        socket!.listen((data) {
          socketNotifier!.notifyData(data);
        }, onError: (err) {
          socketNotifier!.notifyError(Close(err.toString(), 1005));
        }, onDone: () {
          _setStatus(ConnectionStatus.closed);
          socketNotifier!
              .notifyClose(Close("Connection Closed", socket!.closeCode));
        }, cancelOnError: true);
        return;
      } on SocketException catch (e) {
        Get.log("[BaseWebSocket] connected error SocketException");
        _setStatus(ConnectionStatus.closed);
        socketNotifier!
            .notifyError(Close(e.osError!.message, e.osError!.errorCode));
        return;
      }
    }
    return;
  }

  void close([int? status, String? reason]) {
    Get.log("[BaseWebSocket] close");
    _setStatus(ConnectionStatus.closing);
    socket?.close(status, reason);
    _setStatus(ConnectionStatus.closed);
  }

  @Deprecated(
      "尽量不要调用这个方法，请使用[close([int? status, String? reason])]来关闭连接，GetX会自动销毁")
  void dispose([int? status, String? reason]) {
    Get.log("[BaseWebSocket] dispose");
    close(status, reason);
    socketNotifier!.dispose();
    socketNotifier = null;
    isDisposed = true;
  }

  @Deprecated("请不要使用Json进行消息发送，使用[send(dynamic data)]方法来发送消息")
  void emit(String event, dynamic data) {
    send(jsonEncode({"type": event, "data": data}));
  }

  void on(String event, MessageSocket message) {
    socketNotifier!.addEvents(event, message);
  }

  void onClose(CloseSocket fn) {
    socketNotifier!.addCloses(fn);
  }

  void onError(CloseSocket fn) {
    socketNotifier!.addErrors(fn);
  }

  void onMessage(MessageSocket fn) {
    socketNotifier!.addMessages((msg) {
      _setStatus(ConnectionStatus.received);
      fn(msg);
    });
  }

  void send(dynamic data) async {
    Get.log("[BaseWebSocket] send");
    if (connectionStatus == ConnectionStatus.closed) {
      _setStatus(ConnectionStatus.sending);
      await connect();
    }

    if (socket != null) {
      Get.log("[BaseWebSocket] RealSend");
      _setStatus(ConnectionStatus.sended);
      socket!.add(data);
    }
  }

  void _setStatus(ConnectionStatus status) {
    connectionStatus = status;
    socketStatus(status);
  }

  @Deprecated("请不要使用自签名的证书，请使用[connect]方法")
  Future<WebSocket> _connectForSelfSignedCert(String url) async {
    try {
      var r = Random();
      var key = base64.encode(List<int>.generate(8, (_) => r.nextInt(255)));
      var client = HttpClient(context: SecurityContext());
      client.badCertificateCallback = (cert, host, port) {
        Get.log(
            "BaseWebSocket: Allow self-signed certificate => $host:$port. ");
        return true;
      };

      var request = await client.getUrl(Uri.parse(url))
        ..headers.add("Connection", "Upgrade")
        ..headers.add("Upgrade", "websocket")
        ..headers.add("Cache-Control", "no-cache")
        ..headers.add("Sec-WebSocket-Version", "13")
        ..headers.add("Sec-WebSocket-Key", key.toLowerCase());

      var response = await request.close();
      // ignore: close_sinks
      var socket = await response.detachSocket();
      var webSocket = WebSocket.fromUpgradedSocket(
        socket,
        serverSide: false,
      );

      return webSocket;
    } on Exception catch (_) {
      rethrow;
    }
  }
}
