/*
 * Copyright (c) 田梓萱[小草林] 2021-2024.
 * All Rights Reserved.
 * All codes are protected by China's regulations on the protection of computer software, and infringement must be investigated.
 * 版权所有 (c) 田梓萱[小草林] 2021-2024.
 * 所有代码均受中国《计算机软件保护条例》保护，侵权必究.
 */

import "package:flutter/material.dart";

class OtimizedListView<T> extends StatelessWidget {
  final List<T> list;
  final Axis scrollDirection;
  final bool reverse;
  final ScrollController? controller;
  final bool? primary;
  final ScrollPhysics? physics;
  final bool shrinkWrap;
  final Widget onEmpty;
  final int length;
  final Widget Function(BuildContext context, ValueKey key, T item) builder;

  const OtimizedListView({
    super.key,
    required this.list,
    required this.builder,
    this.scrollDirection = Axis.vertical,
    this.reverse = false,
    this.controller,
    this.primary,
    this.physics,
    this.onEmpty = const SizedBox.shrink(),
    this.shrinkWrap = false,
  }) : length = list.length;

  @override
  Widget build(BuildContext context) {
    if (list.isEmpty) return onEmpty;

    return CustomScrollView(
      controller: controller,
      reverse: reverse,
      scrollDirection: scrollDirection,
      primary: primary,
      physics: physics,
      shrinkWrap: shrinkWrap,
      slivers: <Widget>[
        SliverList(
          delegate: SliverChildBuilderDelegate(
            (context, i) {
              final item = list[i];
              final key = ValueKey(item);
              return builder(context, key, item);
            },
            childCount: list.length,
            addAutomaticKeepAlives: true,
            findChildIndexCallback: (key) {
              return list.indexWhere((m) => m == (key as ValueKey<T>).value);
            },
          ),
        ),
      ],
    );
  }
}
